@@ -34,7 +34,7 @@ import queue
 import clang_native
 import jsrun
 import line_endings
-from tools.shared import EMCC, EMXX, DEBUG, EMCONFIGURE, EMCMAKE
+from tools.shared import EMCC, EMXX, DEBUG
 from tools.shared import get_canonical_temp_dir, path_from_root
 from tools.utils import MACOS, WINDOWS, read_file, read_binary, write_binary, exit_with_error
 from tools.settings import COMPILE_TIME_SETTINGS
@@ -85,6 +85,8 @@ WEBIDL_BINDER = shared.bat_suffix(path_from_root('tools/webidl_binder'))
 
 EMBUILDER = shared.bat_suffix(path_from_root('embuilder'))
 EMMAKE = shared.bat_suffix(path_from_root('emmake'))
+EMCMAKE = shared.bat_suffix(path_from_root('emcmake'))
+EMCONFIGURE = shared.bat_suffix(path_from_root('emconfigure'))
 EMRUN = shared.bat_suffix(shared.path_from_root('emrun'))
 WASM_DIS = Path(building.get_binaryen_bin(), 'wasm-dis')
 LLVM_OBJDUMP = os.path.expanduser(shared.build_llvm_tool_path(shared.exe_suffix('llvm-objdump')))
@@ -444,30 +446,26 @@ def also_with_nodefs_both(func):
 
 def with_all_fs(func):
   @wraps(func)
-  def metafunc(self, fs, *args, **kwargs):
+  def metafunc(self, wasmfs, fs, *args, **kwargs):
     if DEBUG:
       print('parameterize:fs=%s' % (fs))
+    if wasmfs:
+      self.setup_wasmfs_test()
     if fs == 'nodefs':
       self.setup_nodefs_test()
     elif fs == 'rawfs':
       self.setup_noderawfs_test()
-    elif fs == 'wasmfs':
-      self.setup_wasmfs_test()
-    elif fs == 'wasmfs_rawfs':
-      self.setup_wasmfs_test()
-      self.setup_noderawfs_test()
     else:
       self.emcc_args += ['-DMEMFS']
       assert fs is None
     func(self, *args, **kwargs)
 
-  # TODO(sbc): rather than treat WASMFS as orthogonal we should
-  # probably make it combinatorial with nodefs and noderawfs.
-  parameterize(metafunc, {'': (None,),
-                          'nodefs': ('nodefs',),
-                          'rawfs': ('rawfs',),
-                          'wasmfs': ('wasmfs',),
-                          'wasmfs_rawfs': ('wasmfs_rawfs',)})
+  parameterize(metafunc, {'': (False, None,),
+                          'nodefs': (False, 'nodefs',),
+                          'rawfs': (False, 'rawfs',),
+                          'wasmfs': (True, None,),
+                          'wasmfs_nodefs': (True, 'nodefs',),
+                          'wasmfs_rawfs': (True, 'rawfs',)})
   return metafunc
 
 
@@ -2038,7 +2036,7 @@ class RunnerCore(unittest.TestCase, metaclass=RunnerMeta):
         os.path.join('third_party', 'poppler'),
         [os.path.join('utils', 'pdftoppm.o'), os.path.join('utils', 'parseargs.o'), os.path.join('poppler', '.libs', 'libpoppler.a')],
         env_init=env_init,
-        configure_args=['--disable-libjpeg', '--disable-libpng', '--disable-poppler-qt', '--disable-poppler-qt4', '--disable-cms', '--disable-cairo-output', '--disable-abiword-output', '--disable-shared'])
+        configure_args=['--disable-libjpeg', '--disable-libpng', '--disable-poppler-qt', '--disable-poppler-qt4', '--disable-cms', '--disable-cairo-output', '--disable-abiword-output', '--disable-shared', '--host=wasm32-emscripten'])
 
     return poppler + freetype
 
@@ -2368,25 +2366,25 @@ class BrowserCore(RunnerCore):
       time.sleep(5)
       print('(moving on..)')
 
-  def compile_btest(self, filename, args, reporting=Reporting.FULL):
+  def compile_btest(self, filename, emcc_args, reporting=Reporting.FULL):
     # Inject support code for reporting results. This adds an include a header so testcases can
     # use REPORT_RESULT, and also adds a cpp file to be compiled alongside the testcase, which
     # contains the implementation of REPORT_RESULT (we can't just include that implementation in
     # the header as there may be multiple files being compiled here).
     if reporting != Reporting.NONE:
       # For basic reporting we inject JS helper funtions to report result back to server.
-      args += ['--pre-js', test_file('browser_reporting.js')]
+      emcc_args += ['--pre-js', test_file('browser_reporting.js')]
       if reporting == Reporting.FULL:
         # If C reporting (i.e. the REPORT_RESULT macro) is required we
         # also include report_result.c and force-include report_result.h
         self.run_process([EMCC, '-c', '-I' + TEST_ROOT,
-                          test_file('report_result.c')] + self.get_emcc_args(compile_only=True) + (['-fPIC'] if '-fPIC' in args else []))
-        args += ['report_result.o', '-include', test_file('report_result.h')]
+                          test_file('report_result.c')] + self.get_emcc_args(compile_only=True) + (['-fPIC'] if '-fPIC' in emcc_args else []))
+        emcc_args += ['report_result.o', '-include', test_file('report_result.h')]
     if EMTEST_BROWSER == 'node':
-      args.append('-DEMTEST_NODE')
+      emcc_args.append('-DEMTEST_NODE')
     if not os.path.exists(filename):
       filename = test_file(filename)
-    self.run_process([compiler_for(filename), filename] + self.get_emcc_args() + args)
+    self.run_process([compiler_for(filename), filename] + self.get_emcc_args() + emcc_args)
 
   def btest_exit(self, filename, assert_returncode=0, *args, **kwargs):
     """Special case of `btest` that reports its result solely via exiting
@@ -2404,20 +2402,20 @@ class BrowserCore(RunnerCore):
 
   def btest(self, filename, expected=None,
             post_build=None,
-            args=None, url_suffix='', timeout=None,
+            emcc_args=None, url_suffix='', timeout=None,
             extra_tries=1,
             reporting=Reporting.FULL,
             output_basename='test'):
     assert expected, 'a btest must have an expected output'
-    if args is None:
-      args = []
-    args = args.copy()
+    if emcc_args is None:
+      emcc_args = []
+    emcc_args = emcc_args.copy()
     filename = find_browser_test_file(filename)
     outfile = output_basename + '.html'
-    args += ['-o', outfile]
-    # print('all args:', args)
+    emcc_args += ['-o', outfile]
+    # print('all args:', emcc_args)
     utils.delete_file(outfile)
-    self.compile_btest(filename, args, reporting=reporting)
+    self.compile_btest(filename, emcc_args, reporting=reporting)
     self.assertExists(outfile)
     if post_build:
       post_build()
